#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "vtk.hh"

template <class VertexBasis, class CellBasis, class Vector, class ScalarVector>
void writeVtk(VertexBasis const &vertexBasis, Vector const &displacement,
              ScalarVector const &logState, CellBasis const &cellBasis,
              ScalarVector const &stress, std::string const &filename) {
  Dune::VTKWriter<typename VertexBasis::GridView> writer(
      vertexBasis.getGridView());

  auto const displacementPointer =
      Dune::make_shared<VTKBasisGridFunction<VertexBasis, Vector> const>(
          vertexBasis, displacement, "displacement");
  writer.addVertexData(displacementPointer);

  auto const logStatePointer =
      Dune::make_shared<VTKBasisGridFunction<VertexBasis, ScalarVector> const>(
          vertexBasis, logState, "logState");
  writer.addVertexData(logStatePointer);

  auto const vonmisesPointer =
      Dune::make_shared<VTKBasisGridFunction<CellBasis, ScalarVector> const>(
          cellBasis, stress, "stress");
  writer.addCellData(vonmisesPointer);

  writer.write(filename.c_str());
}

#include "vtk_tmpl.cc"
